library(propr)
data(caneToad.counts)
counts <- caneToad.counts[,1:1005]

blocks2combs <- function(counts, nblocks){
  
  ngroup <- rep(1:nblocks, each = 100)
  leftover <- ncol(counts) %% nblocks
  if(leftover > 0) ngroup <- c(ngroup, rep(nblocks + 1, leftover))
  split <- split(1:ncol(counts), ngroup)
  
  combs <- expand.grid(1:length(split), 1:length(split))
  combs <- t(apply(combs, 1, sort))
  combs <- unique(combs) 
  combs <- combs[combs[,1] != combs[,2],]
  
  return(combs)
}

#' Parallelize \code{propr}
#' 
#' This function parallelizes \code{\link{propr}}.
#' 
#' @inheritParams propr
#' @param nblocks An integer. Sets how many times to split the data.
#'  This typically results in \code{(nblocks+1)(nblocks)/2} jobs.
#' @param ncores An integer. The number of cores to used.
#'  Argument passed to \code{doMC::registerDoMC}.
#' @param dir A string. The directory to which to save the results.
#'  If \code{dir = NA}, a single matrix is returned (RAM-hungry mode).
#'  Otherwise, each job is saved locally (RAM-saving mode).
#'  
#' @return This function returns a proportionality matrix.
#'  This function does not return an S4 object.
#'  
#' @export
propr.dopar <- function(counts, metric = c("rho", "phi", "phs", "cor", "vlr"),
                        ivar = "clr", symmetrize = FALSE, alpha, p = 100,
                        nblocks = ncol(counts) %/% 100, ncores = 4, dir = NA){
  
  combs <- blocks2combs(counts, nblocks)
  
  doMC::registerDoMC(cores = ncores)
  `%dopar%` <- foreach::`%dopar%`
  
  RES <- foreach::foreach(i = 1:nrow(combs)) %dopar% {
    
    batch1 <- split[[combs[i,1]]]
    batch2 <- split[[combs[i,2]]]
    rho.i <- propr(counts, metric = metric, ivar = viar, symmetrize = symmetrize,
                   alpha = alpha, p = p, select = c(batch1, batch2))
    
    if(is.na(dir)){
      
      rho.i@matrix
      
    }else{
      
      file1 <- paste0("job-", combs[i,1], "+", combs[i,2], ".csv")
      file2 <- paste0("job-", combs[i,1], "+", combs[i,2], ".RData")
      write.csv(rho.i@matrix, file = file1)
      save(rho.i, file = file2)
    }
  }
  
  if(!is.na(dir)) return(NULL)
  QUILT <- matrix(0, ncol(counts), ncol(counts))
  for(i in 1:nrow(combs)){
    
    # Fill final matrix with each job
    batch1 <- split[[combs[i,1]]]
    batch2 <- split[[combs[i,2]]]
    patch.i <- c(batch1, batch2)
    QUILT[patch.i, patch.i] <- RES[[i]]
  }
  
  return(QUILT)
}

propd.dopar <- function(){
  
}
